// anim.c - for gameland - Stephen Stair
// sprite animation routines
// path management...

#include "gameland.h"

sprinfo sprites[128];
u16 numsprites;
u16 tilesmoved;

#define SprType_regular	0
#define SprType_char	1
#define SprType_car		2
#define SprType_RotScale	3
#define SprType_userman 4

void ClearOam() {
	u16 i;
	for(i=0;i<128;i++) { sprites[i].visible=0; sprites[i].id=0; }
	numsprites=0;
}

void AddSprite(u8 type, u8 id, u8 priority,u8 visible, u32 * dat) {
	u32 i,j;
	if(numsprites==128) return;
	sprinfo myspr;
	for(i=0;i<numsprites;i++) {
		if(sprites[i].id==id) return; // can't add same ID twice
	}
	myspr.id=id;
	myspr.priority=priority;
	myspr.spr=dat;
	myspr.type=type;
	myspr.visible=visible;
	for(i=0;i<numsprites;i++) {
		if(sprites[i].priority>priority) break;
	}
	// i is index to insert into
	for(j=numsprites+1;j>i;j--) if(j<127) sprites[j-1]=sprites[j-2];
	sprites[i]=myspr;
	numsprites++;
	tilesmoved=1;
	// inserted successfully.
}
void DeleteSprite(u8 id) {
	u8 i,j;
	for(i=0;i<numsprites;i++) {
		if(sprites[i].id==id) break; 
	}
	if(i==numsprites) return; // id doesn't exist
	// i is index to delete into
	numsprites--;
	for(j=i;j<numsprites;j++) sprites[j]=sprites[j+1];
	sprites[j+1].id=0;
	sprites[j+1].visible=0;
	tilesmoved=1;
}
void SetVisible(u8 id, u8 visible) {
	u8 i;
	for(i=0;i<numsprites;i++) {
		if(sprites[i].id==id) {sprites[i].visible=visible;tilesmoved=1; return;}
	}
}
void SetPriority( u8 id, u8 priority) {
	u8 i;
	sprinfo myspr;
	for(i=0;i<numsprites;i++) {
		if(sprites[i].id==id) break;
	}
	if(i==numsprites) return;
	myspr=sprites[i];
	DeleteSprite(id);
	AddSprite(myspr.type,id,priority,myspr.visible,myspr.spr);
	// easy way to do it :P

}
const u16 sizelut[] = {
	1,4,16,64,2,4,8,32,2,4,8,32
};

u32 drawStaticSpr(staticspr * ss, u32 starttile) {
	u16 numtiles;
	u16 i;
	numtiles=sizelut[ss->size]*2;
	for(i=0;i<numtiles*32;i++) VRAM[0x010000/2+starttile*16+i]=ss->tiledata[i];
	return numtiles;
}

u32 drawCharSpr(charinfo * ci, u32 starttile) {
	// numtiles=4 (8 counting 16color tiles)
	u16 *srcdata;
	u16 i;
	u16 srctile;
	srcdata=ci->chdata->anim->sprdata; // initial pointer
	if(ci->state==0) {
		srctile=ci->direction;
	} else if(ci->state==1) {
		i=ci->chdata->anim->walk[ci->direction/2].length;
		if(i>0) {
			srctile=ci->chdata->anim->walk[ci->direction/2].starttile+((ci->animframe/2)%i);
		} else {
			srctile=ci->chdata->anim->walk[ci->direction/2].starttile;
		}
	} else {
		i=ci->chdata->anim->other[ci->state-2].length;
		if(i>0) {
			srctile=ci->chdata->anim->other[ci->state-2].starttile+((ci->animframe/2)%i);
		} else {
			srctile=ci->chdata->anim->other[ci->state-2].starttile;
		}
	}
	for(i=0;i<4*32;i++) VRAM[0x010000/2+starttile*16+i]=srcdata[srctile*128+i];
	return 8;
}



// call once a vblank
void UpdateOam() {
	u8 i;
	s16 x,y;
	u32 vramtilenum;
	u8 curspr;
	charinfo * ci;
	staticspr * ss;
	vramtilenum=0;
	curspr=0;
	for(i=0;i<numsprites;i++) {
		if(sprites[i].visible) {
			switch(sprites[i].type) {
			case SprType_regular:
				ss=(staticspr *) sprites[i].spr;
				if(tilesmoved) {
					sprites[i].starttilenum=vramtilenum;
					vramtilenum+=drawStaticSpr(ss,vramtilenum);
				}
				OAM[curspr*4]=((ss->size/4)<<14) | 0x2000 | (ss->y & 255);
				OAM[curspr*4+1]=((ss->size&3)<<14) | (ss->x & 511);
				OAM[curspr*4+2]=sprites[i].starttilenum;

				break;
			case SprType_char:
				ci=(charinfo *) sprites[i].spr;
				x=ci->loc.c*16-mpx;
				y=ci->loc.r*16-7-mpy;
				if(ci->state==1) {
					switch(ci->direction/2) {// we're moving so character is in slightly different spot
					case 0:
						y-=ci->animframe;
						break;
					case 1:
						x+=ci->animframe;
						break;
					case 2:
						y+=ci->animframe;
						break;
					case 3:
						x-=ci->animframe;
						break;
					}
				}
				if(x>240 || y>160 || x<-16 || y<-16) {
					curspr--;
					break;
				}
				// we assume all other states/animations are stationary

				if(tilesmoved) {
					sprites[i].starttilenum=vramtilenum;
					vramtilenum+=drawCharSpr(ci,vramtilenum);
				} else //if(ci->needupdate || ci->state) {
					drawCharSpr(ci,sprites[i].starttilenum);
				//}
				OAM[curspr*4]= 0x2000 | (y & 255);
				OAM[curspr*4+1]=0x4000 | (x & 511);
				OAM[curspr*4+2]=sprites[i].starttilenum;
				

				break;
			case SprType_car:
				break;
			case SprType_RotScale:
				break;
			case SprType_userman:
				ss=(staticspr *) sprites[i].spr;
				OAM[curspr*4]=((ss->size/4)<<14) | 0x2000 | (ss->y & 255);
				OAM[curspr*4+1]=((ss->size&3)<<14) | (ss->x & 511);
				OAM[curspr*4+2]=ss->spritenum;
				break;


			}
		}
		curspr++;
	}	
	for(i=curspr;i<128;i++) {
		OAM[i*4]=161; // clear
	}
	tilesmoved=0;
}

