// gbadefines 
// defines of GBA registers from NO$'s gbatek for easy access in C/C++
// 2003 Stephen Stair
// v1.0 2-11-03 - Began file,

#include "gba.h"


//extra stuff
#define		INTRADDR	(*((u32 volatile *) 0x03007FFC))
#define		VRAM		((u16 *) 0x06000000)
#define		PAL			((u16 *) 0x05000000)
#define		OAM			((u16 *) 0x07000000)


//LCD I/O Registers
#define		DISPCNT		(*((u16 volatile *) 0x04000000))
#define		DISPSTAT	(*((u16 volatile *) 0x04000004))
#define		VCOUNT		(*((u16 volatile *) 0x04000006))
#define		BG0CNT		(*((u16 volatile *) 0x04000008))
#define		BG1CNT		(*((u16 volatile *) 0x0400000A))
#define		BG2CNT		(*((u16 volatile *) 0x0400000C))
#define		BG3CNT		(*((u16 volatile *) 0x0400000E))
#define		BG0HOFS		(*((u16 volatile *) 0x04000010))
#define		BG0VOFS		(*((u16 volatile *) 0x04000012))
#define		BG1HOFS		(*((u16 volatile *) 0x04000014))
#define		BG1VOFS		(*((u16 volatile *) 0x04000016))
#define		BG2HOFS		(*((u16 volatile *) 0x04000018))
#define		BG2VOFS		(*((u16 volatile *) 0x0400001A))
#define		BG3HOFS		(*((u16 volatile *) 0x0400001C))
#define		BG3VOFS		(*((u16 volatile *) 0x0400001E))
#define		BG2PA		(*((u16 volatile *) 0x04000020))
#define		BG2PB		(*((u16 volatile *) 0x04000022))
#define		BG2PC		(*((u16 volatile *) 0x04000024))
#define		BG2PD		(*((u16 volatile *) 0x04000026))
#define		BG2X		(*((u32 volatile *) 0x04000028))
#define		BG2Y		(*((u32 volatile *) 0x0400002C))
#define		BG3PA		(*((u16 volatile *) 0x04000030))
#define		BG3PB		(*((u16 volatile *) 0x04000032))
#define		BG3PC		(*((u16 volatile *) 0x04000034))
#define		BG3PD		(*((u16 volatile *) 0x04000036))
#define		BG3X		(*((u32 volatile *) 0x04000038))
#define		BG3Y		(*((u32 volatile *) 0x0400003C))
#define		WIN0H		(*((u16 volatile *) 0x04000040))
#define		WIN1H		(*((u16 volatile *) 0x04000042))
#define		WIN0V		(*((u16 volatile *) 0x04000044))
#define		WIN1V		(*((u16 volatile *) 0x04000046))
#define		WININ		(*((u16 volatile *) 0x04000048))
#define		WINOUT		(*((u16 volatile *) 0x0400004A))
#define		MOSAIC		(*((u16 volatile *) 0x0400004C))
#define		BLDCNT		(*((u16 volatile *) 0x04000050))
#define		BLDALPHA	(*((u16 volatile *) 0x04000052))
#define		BLDY		(*((u16 volatile *) 0x04000054))

//Sound Registers
#define		SOUND1CNT_L	(*((u16 volatile *) 0x04000060))
#define		SOUND1CNT_H	(*((u16 volatile *) 0x04000062))
#define		SOUND1CNT_X	(*((u16 volatile *) 0x04000064))
#define		SOUND2CNT_L	(*((u16 volatile *) 0x04000068))
#define		SOUND2CNT_H	(*((u16 volatile *) 0x0400006C))
#define		SOUND3CNT_L	(*((u16 volatile *) 0x04000070))
#define		SOUND3CNT_H	(*((u16 volatile *) 0x04000072))
#define		SOUND3CNT_X	(*((u16 volatile *) 0x04000074))
#define		SOUND4CNT_L	(*((u16 volatile *) 0x04000078))
#define		SOUND4CNT_H	(*((u16 volatile *) 0x0400007C))
#define		SOUNDCNT_L	(*((u16 volatile *) 0x04000080))
#define		SOUNDCNT_H	(*((u16 volatile *) 0x04000082))
#define		SOUNDCNT_X	(*((u16 volatile *) 0x04000084))
#define		SOUNDBIAS	(*((u16 volatile *) 0x04000088))
#define		WAVE_RAM	((u16 volatile *) 0x04000090)
#define		FIFO_A		(*((u32 volatile *) 0x040000A0))
#define		FIFO_B		(*((u32 volatile *) 0x040000A4))
 
//DMA Transfer Channels
#define		DMA0SAD		(*((u32 volatile *) 0x040000B0))
#define		DMA0DAD		(*((u32 volatile *) 0x040000B4))
#define		DMA0CNT_L	(*((u16 volatile *) 0x040000B8))
#define		DMA0CNT_H	(*((u16 volatile *) 0x040000BA))
#define		DMA1SAD		(*((u32 volatile *) 0x040000BC))
#define		DMA1DAD		(*((u32 volatile *) 0x040000C0))
#define		DMA1CNT_L	(*((u16 volatile *) 0x040000C4))
#define		DMA1CNT_H	(*((u16 volatile *) 0x040000C6))
#define		DMA2SAD		(*((u32 volatile *) 0x040000C8))
#define		DMA2DAD		(*((u32 volatile *) 0x040000CC))
#define		DMA2CNT_L	(*((u16 volatile *) 0x040000D0))
#define		DMA2CNT_H	(*((u16 volatile *) 0x040000D2))
#define		DMA3SAD		(*((u32 volatile *) 0x040000D4))
#define		DMA3DAD		(*((u32 volatile *) 0x040000D8))
#define		DMA3CNT_L	(*((u16 volatile *) 0x040000DC))
#define		DMA3CNT_H	(*((u16 volatile *) 0x040000DE))
 
//Timer Registers
#define		TM0CNT_L	(*((u16 volatile *) 0x04000100))
#define		TM0CNT_H	(*((u16 volatile *) 0x04000102))
#define		TM1CNT_L	(*((u16 volatile *) 0x04000104))
#define		TM1CNT_H	(*((u16 volatile *) 0x04000106))
#define		TM2CNT_L	(*((u16 volatile *) 0x04000108))
#define		TM2CNT_H	(*((u16 volatile *) 0x0400010A))
#define		TM3CNT_L	(*((u16 volatile *) 0x0400010C))
#define		TM3CNT_H	(*((u16 volatile *) 0x0400010E))

//Serial Communication (1)
#define		SIODATA32	(*((u32 volatile *) 0x04000120))
#define		SIOMULTI0	(*((u16 volatile *) 0x04000120))
#define		SIOMULTI1	(*((u16 volatile *) 0x04000122))
#define		SIOMULTI2	(*((u16 volatile *) 0x04000124))
#define		SIOMULTI3	(*((u16 volatile *) 0x04000126))
#define		SIOCNT		(*((u16 volatile *) 0x04000128))
#define		SIOMLT_SEND	(*((u16 volatile *) 0x0400012A))
#define		SIODATA8	(*((u16 volatile *) 0x0400012A))
 
//Keypad Input
#define		KEYINPUT	(*((u16 volatile *) 0x04000130))
#define		KEYCNT		(*((u16 volatile *) 0x04000132))
 
//Serial Communication (2)
#define		RCNT		(*((u16 volatile *) 0x04000134))
#define		JOYCNT		(*((u16 volatile *) 0x04000140))
#define		JOY_RECV	(*((u32 volatile *) 0x04000150))
#define		JOY_TRANS	(*((u32 volatile *) 0x04000154))
#define		JOYSTAT		(*((u16 volatile *) 0x04000158))

//Interrupt, Waitstate, and Power-Down Control
#define		IE			(*((u16 volatile *) 0x04000200))
#define		IF			(*((u16 volatile *) 0x04000202))
#define		WAITCNT		(*((u16 volatile *) 0x04000204))
#define		IME			(*((u16 volatile *) 0x04000208))


// allow old names: (for compatiblity with some older routines I threw in)
#define		BLDMOD		BLDCNT
#define		COLEV		BLDALPHA
#define		COLY		BLDY
#define		TM0D		TM0CNT_L
#define		TM0CNT		TM0CNT_H
#define		TM1D		TM1CNT_L
#define		TM1CNT		TM1CNT_H
#define		TM2D		TM2CNT_L
#define		TM2CNT		TM2CNT_H
#define		TM3D		TM3CNT_L
#define		TM3CNT		TM3CNT_H
#define		P1			KEYINPUT
#define		P1CNT		KEYCNT


// ok that's done!
