#include "gameland.h"
// mapscrolling routines - Stephen Stair 2003

void SetupMode() {
	u32 i;
	for(i=0;i<512;i++) PAL[i]=0; //blackout palette
	BG0CNT=0x1F81;// char block 0, screen block 31
	DISPCNT=0x1140; // mode 0 with bg0 and sprites
}


tileset * curtileset;
mapdata * curmap;
void ResetTileSet() {
	curtileset=NULL;
}
void LoadTileSet(tileset * tset) {
	u32 i;
	curtileset=tset;
	for(i=0;i<tset->numtiles*4*32;i++) VRAM[i]=(tset->tiledata)[i];
	for(i=0;i<tset->numpals;i++) PAL[i]=(tset->paldata)[i];
}

u32 mpx,mpy; // mpx,mpy coordinates of top left screen in map

// when we talk about line here, it's the 16-pixel wide one.

#define map(x,y) VRAM[0x0f800/2+(x)+(y)*32]
void SetMapTile(u8 x, u8 y, u16 t) {
	u16 tl;
	u8 hvflp;
	tl=t & 1023;
	hvflp=t / 1024;
	t=tl*4+hvflp*1024;
	switch(hvflp) {
	case 0: // normal
		map(x*2,y*2)=t;
		map(x*2+1,y*2)=t+1;
		map(x*2,y*2+1)=t+2;
		map(x*2+1,y*2+1)=t+3;
		break;
	case 1: // hflip
		map(x*2,y*2)=t+1;
		map(x*2+1,y*2)=t;
		map(x*2,y*2+1)=t+3;
		map(x*2+1,y*2+1)=t+2;
		break;
	case 2: // vflip
		map(x*2,y*2)=t+2;
		map(x*2+1,y*2)=t+3;
		map(x*2,y*2+1)=t;
		map(x*2+1,y*2+1)=t+1;
		break;
	case 3: // both
		map(x*2,y*2)=t+3;
		map(x*2+1,y*2)=t+2;
		map(x*2,y*2+1)=t+1;
		map(x*2+1,y*2+1)=t;
		break;
	}
}
void ms_RedrawFullScreen() {
	u32 x,y;
	for(y=mpy/16;y<mpy/16+11;y++) {
		for(x=mpx/16;x<mpx/16+16;x++) {
			SetMapTile(x%16,y%16,(curmap->map)[x+y*curmap->width]);
		}
	}

}
void ms_RedrawTopLine() {
ms_RedrawFullScreen();
}
void ms_RedrawBottomLine() {
ms_RedrawFullScreen();
}
void ms_RedrawRightLine() {
ms_RedrawFullScreen();
}
void ms_RedrawLeftLine() {
ms_RedrawFullScreen();
}
void ms_SetScrollPos() {
	BG0HOFS=mpx;
	BG0VOFS=mpy;

}


void LoadMap(mapdata * map,s32 x,s32 y) {
	curmap=map;
	x=x-120;
	y=y-80;	// adjust for center;
	if(x<0) x=0;
	if(y<0) y=0;
	if(x>(map->width*16-241)) x=map->width*16-241;
	if(y>(map->height*16-161)) y=(map->height*16-161);
	mpy=y;
	mpx=x;
	ms_RedrawFullScreen();
}

void CenterMap(s32 x, s32 y) {
	s32 dx,dy;
	x=x-120;
	y=y-80;	// adjust for center;
	if(x<0) x=0;
	if(y<0) y=0;
	if(x>(curmap->width*16-241)) x=curmap->width*16-241;
	if(y>(curmap->height*16-161)) y=(curmap->height*16-161);
	dx=(x/16)-(mpx/16);
	dy=(y/16)-(mpy/16);
	mpy=y;
	mpx=x;

	switch(dx) {
	case 0: // easy as pie, nothing to do
		break;
	case 1: // x > previous x, so add a row to the right
		ms_RedrawRightLine();
		break;
	case -1:// opposite
		ms_RedrawLeftLine();
		break;
	default: // we're too far away, draw again...
		ms_RedrawFullScreen();
	}

	switch(dy) {
	case 0: // easy as pie, nothing to do
		break;
	case 1: // y > previous y, so add a row to the bottom
		ms_RedrawBottomLine();
		break;
	case -1:// opposite
		ms_RedrawTopLine();
		break;
	default: // we're too far away, draw again...
		ms_RedrawFullScreen();
	}
	ms_SetScrollPos();
}





