#include "gameland.h"



const Module * music[] = {
	&mod_dgintro,&mod_breakf,&mod_carousel,
	&mod_climax2,&mod_crack,&mod_daimondd,
	&mod_dedicate,&mod_enfrance
};



// time functions / variables
u32 systemtime; //. updated every ms by interrupt - accurate to 1ms every 2/3 year
u32 offsetcount;
u32 GetTime(void) {
	return systemtime;
}

// sleep function:
void SleepIntrReturn(void) {
	asm volatile (
		"mov r0,#0x04000000 \n" // reset interrupt flag (IF)
		"add r0,r0,#0x00200 \n"
		"ldr r1,[r0,#2] \n"
		"str r1,[r0,#2] \n"		
		// output:
		: 
		// input:
		:
		// destroy:
		: "r0","r1","r2","r3","r12","lr"
	);
}
void SleepKey(u32 KeyMask){ // GBA hibernates until key in keymask is pressed.
	u32 oldIE,oldIME,oldDISPCNT,oldINTRADDR,oldKEYCNT;

	if(!(KeyMask&0x03FF)) return; // if no keys in mask - don't go - we'll be locked forever

	oldIE=IE;
	oldIME=IME;
	oldKEYCNT=KEYCNT;
	oldDISPCNT=DISPCNT;
	oldINTRADDR=INTRADDR;

	IME=0; // disable interrupts while we sort this mess out
	IE=0x1000; // enable key interrupt
	INTRADDR=(u32) &SleepIntrReturn; // temp return function
	DISPCNT=0x0080; // turn off display
	KEYCNT=0xC000+KeyMask && 0x0FFF; // add the keymask to the interrupt mix.
		// in AND mode, must press ALL keys to return from sleep mode!
	IME=1; // ready to roll!

	asm volatile (
		"swi #0x030000 \n" // 32bit swi
		// output:
		: 
		// input:
		: 
		// destroy:
		: "r0","r1","r2","r3","r12","lr"
	);
	IME=0; // disable until we get going..
	IE=oldIE;
	INTRADDR=oldINTRADDR;
	DISPCNT=oldDISPCNT;
	KEYCNT=oldKEYCNT;
	IME=oldIME;
	// ready to continue with life!
}






const u8 TextSin[] ={ // 65 entries
  0,  1,  2,  3,  4,  5,  7,  8,
  9,  10,  11,  12,  13,  15,  16,  17,
  18,  19,  20,  21,  22,  23,  24,  25,
  26,  27,  28,  29,  30,  31,  32,  33,
  33,  34,  35,  36,  37,  37,  38,  39,
  39,  40,  41,  41,  42,  42,  43,  43,
  44,  44,  45,  45,  45,  46,  46,  46,
  47,  47,  47,  47,  47,  47,  47,  47,
  48
};
const u8 pntrsin[] = { // 32 entries
  0,  0,  1,  2,  3,  3,  4,  5,
  5,  6,  6,  7,  7,  7,  7,  7,
  7,  7,  7,  7,  7,  7,  6,  6,
  5,  5,  4,  3,  3,  2,  1,  0
};


int GetCharLength(u32 font, char fs);
int GetWordLength(u32 font, char * fs);
int TextCharOut(s32 x, s32 y,u32 font, char  fs, u32 color);


#define t_basetile 16
// text routines (text box 32 high, 240 wide)
void TextStart() {
	// load tiles+pals
	PlayGBCSound(2);

	u32 i;
	for(i=240;i<255;i++) PAL[i]=framestuff_pal[i];
	for(i=0;i<32;i++) VRAM[t_basetileram/2+i]=0; // transparent tile, tile#16
	for(i=0;i<32*9;i++) VRAM[(t_basetileram+0x40)/2+i]=((u16 *)framestuff_tile)[i]; // next 9 tiles
	TextClear();

	BG1HOFS=0;
	BG1VOFS=0;
	BG1CNT=0x1E8C;
	DISPCNT |= 0x0200; // enable bg1
	// use nice thingy to show it
	for(i=0;i<33;i++) {
		WaitVblank();
		BG1VOFS=TextSin[i*2];
	}

}
staticspr textspr;
void TextDisp(char * txt, u16 snd) {
	if(snd>0) snd=snd+3;
	s32 x,y;
	u32 i;
	u8 newword;
	x=0;
	y=0;
	i=0;
	newword=1;
	if(txt[i]==0) return;
	do {
		WaitVblank();
		if(newword==1) {
			if(x+GetWordLength(0,&txt[i])>224) {x=0;y=y+8;}
			newword=0;
		}
		if(txt[i]==' ') {newword=1;PlayGBCSound(snd);}
		x+=TextCharOut(x,y,0,txt[i],255);
		if(txt[i]=='\n') {x=0;y=y+8;}
		if(x>224) {x=0;y=y+8;}
		
		i++;
	}while(txt[i]!=0 && y<32);
	i=0;
	textspr.size=0;
	textspr.tiledata=(u16 *) &framestuff_tile[64*8];
	textspr.x=226;
	textspr.y=144;
	PAL[510]=0x7fff;
	PAL[511]=0;
	AddSprite(0,255,1,1,(u32 *) &textspr);
	do {
		WaitVblank();
		textspr.y=152-pntrsin[i%32];
		i++;
	} while(!KeyPressed(KEY_A)); // wait for key
	DeleteSprite(255);
}

u8 TextAskYN(char * txt, u16 snd) {
	if(snd>0) snd=snd+3;
	s32 x,y;
	u32 i;
	u8 newword;
	x=0;
	y=0;
	i=0;
	newword=1;
	if(txt[i]==0) return 0;
	do {
		WaitVblank();
		if(newword==1) {
			if(x+GetWordLength(0,&txt[i])>224) {x=0;y=y+8;}
			newword=0;
		}
		if(txt[i]==' ') {newword=1;PlayGBCSound(snd);}
		TextCharOut(x,y,0,txt[i],255);
		x=x+GetCharLength(0,txt[i]);
		if(txt[i]=='\n') {x=0;y=y+8;}
		if(x>224) {x=0;y=y+8;}
		
		i++;
	}while(txt[i]!=0 && y<32);
	y=1;
	newword=1;
	do {
		WaitVblank();
		if(newword==1) {
			for(i=0;i<8*32;i++) VRAM[28*3*32+(t_basetileram+(64*10))/2+i] =0xFEFE; // 254,254
			if(y==1) {
				x=0;
				x+=TextCharOut(x,24,0,'Y',255);
				x+=TextCharOut(x,24,0,'e',255);
				x+=TextCharOut(x,24,0,'s',255);
			} else {
				x=0;
				x+=TextCharOut(x,24,0,'N',255);
				x+=TextCharOut(x,24,0,'o',255);
			}
			newword=0;
		}

		if(KeyPressed(KEY_LEFT | KEY_RIGHT |KEY_UP |KEY_DN| KEY_SELECT)) {
			newword=1;PlayGBCSound(snd);
			y=(y+1)%2;
		}
	} while(!KeyPressed(KEY_A)); // wait for key
	return y;
}


void TextClear() {
	u16 c;
	u32 i;
	for(i=0;i<32*32;i++) {	// clear map
		if(i<32*20) c=t_basetile;
		else if(i==32*20) c=t_basetile+1;
		else if(i<32*20+29) c=t_basetile+5;
		else if(i==32*20+29) c=t_basetile+2;
		else if(i<32*25) {
			c=t_basetile+10+(i%32-1)+(i/32-21)*28;
			if(i%32==0) c=t_basetile+6;
			else if(i%32==29) c=t_basetile+7;
		}
		else if(i==32*25) c=t_basetile+3;
		else if(i<32*25+29) c=t_basetile+8;
		else if(i==32*25+29) c=t_basetile+4;
		else c=t_basetile;
		VRAM[0x0f000/2+i]=c;
	}
	// clear tiles
	for(i=0;i<28*4*32;i++) VRAM[(t_basetileram+(64*10))/2+i] =0xFEFE; // 254,254
}
void TextEnd() {
	u32 i;
	PlayGBCSound(3);
	for(i=32;i<34;i--) {
		WaitVblank();
		BG1VOFS=TextSin[i*2];
	}
	DISPCNT &= 0xFDFF; // disable bg1
}




// character + animation definitions

















// rpg game routines
mapdata * curmap;

u8 rpg_mapfollowchar;

void IntrCntrMap() {// interrupt called routine
	s16 x,y;
	if(rpg_mapfollowchar) {
		x=mainchar.loc.c*16;
		y=mainchar.loc.r*16-7;
		if(mainchar.state==1) {
			switch(mainchar.direction/2) {// we're moving so character is in slightly different spot
			case 0:
				y-=mainchar.animframe;
				break;
			case 1:
				x+=mainchar.animframe;
				break;
			case 2:
				y+=mainchar.animframe;
				break;
			case 3:
				x-=mainchar.animframe;
				break;
			}
		}
		CenterMap(x,y);
	}
}

extern const charinfo otherchars[];

void rpg_changemap(mapdata * md,s32 epnum) {
	u32 i;
	if(epnum>md->eps->numpoints) epnum=-1; // can't do it
	rpg_mapfollowchar=0; // disable while we swap map

	curmap=md;
	mainchar.currentmap=md;

	SetupMode();
	// update mainchar
	if(epnum<0) { // don't update char position
	} else { // otherwise do
		mainchar.loc.r=md->eps->pt[epnum].r;
		mainchar.loc.c=md->eps->pt[epnum].c;
	}
	LoadMap(md,mainchar.loc.c*16+8,mainchar.loc.r*16+8);
	LoadTileSet(md->ts);
	ClearOam();
	for(i=0;i<255;i++) PAL[256+i]=spritedata_pal[i];
	// clear old sprites + set new ones
	AddSprite( 1,1,30,1,(u32 *)&mainchar); // always add main char
	for(i=0;i<2;i++) {
		if(otherchars[i].currentmap==curmap) {
			AddSprite(1,i+5,31,1,(u32 *)&otherchars[i]);
		}
	}
	// add other characters here
	if(mainchar.mystate==1) {
		// find first free direction and point the char out :)
		if(curmap->rpg[(mainchar.loc.r-1)*curmap->width+mainchar.loc.c].blocked==0 && curmap->rpg[(mainchar.loc.r-1)*curmap->width+mainchar.loc.c].event!=rpgevent_mapchg) {
			mainchar.direction=0;
		} else if(curmap->rpg[(mainchar.loc.r)*curmap->width+mainchar.loc.c+1].blocked==0 && curmap->rpg[(mainchar.loc.r)*curmap->width+mainchar.loc.c+1].event!=rpgevent_mapchg) {
			mainchar.direction=2;
		} else if(curmap->rpg[(mainchar.loc.r+1)*curmap->width+mainchar.loc.c].blocked==0 && curmap->rpg[(mainchar.loc.r+1)*curmap->width+mainchar.loc.c].event!=rpgevent_mapchg) {
			mainchar.direction=4;
		} else {
			mainchar.direction=6;
		}
	}
	rpg_mapfollowchar=1; // let interrupt center map for us
}

u16 tempperson;
u32 tempparam;
u32 gamestate;

void rpg_textevt(char * txt) {
	TextStart();
	TextDisp(txt,1);
	TextEnd();
}

void rpg_event(u32 parama,u32 paramb) {

	switch(parama) {
	case 0: // regular event
		if(paramb==gamestate) { // only trigger when we're sure
		switch(gamestate) {
		case 0:
			rpg_textevt("Mom: Someone's left a message for you on the computer! Come down and get it!");
			break;
		case 1:
			rpg_textevt("Mom: If you're going out can you please get me some milk at the store?");
			break;
		case 2:
			rpg_textevt("You got milk.");
			break;
		case 3:
			rpg_textevt("You bought milk.\nThanks for shopping at 1337 Stores, Inc.");
			break;
		case 4:
			rpg_textevt("Thanks for getting milk.  Why don't you go down to the arcade with your friends?");

		}
		gamestate++;
	}
		break;
	case 1:	// policing event - check that you're not in trouble
		if(paramb==gamestate)
			switch(gamestate) {
			case 3: // you have milk, haven't paid...
				rpg_textevt("Security: Where do you think you're going without paying for that?");
				mainchar.direction=0; // push back
				mainchar.animframe=0;
				mainchar.state=1;
				break;

			}
		break;
	case 2:	// you are in trouble!



	case 3: // completely unrelated event:
		switch(paramb) {
		case 0:

			break;
		}

	}
}

void rpg_gameloop() {
	u32 i,a;
	SetupMode();
	rpg_changemap((mapdata *)&aptA2,1);
	KraPlay((Module *)&mod_breakf);
	FadeInBlack(32);
	gamestate=0;
	rpg_event(0,0);

	do {
		WaitVblank();
		// sprite state updating
		if(mainchar.animframe==16) {
			// what were we doing?
			if(mainchar.state==1) {
				// walking
				// do event on destination tile if there is one
				switch(mainchar.direction) {
				case 0:
					mainchar.loc.r--;
					break;
				case 2:
					mainchar.loc.c++;
					break;
				case 4:
					mainchar.loc.r++;
					break;
				case 6:
					mainchar.loc.c--;
				}
			}
			mainchar.state=0;
			if(curmap->rpg[(mainchar.loc.r)*curmap->width+mainchar.loc.c].event==10) {
				rpg_event(curmap->rpg[(mainchar.loc.r)*curmap->width+mainchar.loc.c].person,
					curmap->rpg[(mainchar.loc.r)*curmap->width+mainchar.loc.c].param);
			}
		}

		// input updating
		if(mainchar.state==0) {
			if(KeyDown(KEY_UP)) {
				mainchar.direction=0;
				mainchar.needupdate=1;
				// check dest tile
				if(mainchar.loc.r>0) {
					if(curmap->rpg[(mainchar.loc.r-1)*curmap->width+mainchar.loc.c].blocked==0) {
						mainchar.animframe=0;
						mainchar.state=1;
						if(curmap->rpg[(mainchar.loc.r-1)*curmap->width+mainchar.loc.c].event==rpgevent_mapchg) {
							mainchar.mystate=1; // changing map
							tempperson=curmap->rpg[(mainchar.loc.r-1)*curmap->width+mainchar.loc.c].person;
							tempparam=curmap->rpg[(mainchar.loc.r-1)*curmap->width+mainchar.loc.c].param;
						}

					}
				}
			}
		}
		if(mainchar.state==0) {
			if(KeyDown(KEY_DN)) {
				mainchar.direction=4;
				mainchar.needupdate=1;
				// check dest tile
				if(mainchar.loc.r<curmap->height) {
					if(curmap->rpg[(mainchar.loc.r+1)*curmap->width+mainchar.loc.c].blocked==0) {
						mainchar.animframe=0;
						mainchar.state=1;
						if(curmap->rpg[(mainchar.loc.r+1)*curmap->width+mainchar.loc.c].event==rpgevent_mapchg) {
							mainchar.mystate=1; // changing map
							tempperson=curmap->rpg[(mainchar.loc.r+1)*curmap->width+mainchar.loc.c].person;
							tempparam=curmap->rpg[(mainchar.loc.r+1)*curmap->width+mainchar.loc.c].param;
						}
					}
				}
			}
		}
		if(mainchar.state==0) {
			if(KeyDown(KEY_LEFT)) {
				mainchar.direction=6;
				mainchar.needupdate=1;
				// check dest tile
				if(mainchar.loc.c>0) {
					if(curmap->rpg[(mainchar.loc.r)*curmap->width+mainchar.loc.c-1].blocked==0) {
						mainchar.animframe=0;
						mainchar.state=1;
						if(curmap->rpg[(mainchar.loc.r)*curmap->width+mainchar.loc.c-1].event==rpgevent_mapchg) {
							mainchar.mystate=1; // changing map
							tempperson=curmap->rpg[(mainchar.loc.r)*curmap->width+mainchar.loc.c-1].person;
							tempparam=curmap->rpg[(mainchar.loc.r)*curmap->width+mainchar.loc.c-1].param;
						}
					}
				}
			}
		}
		if(mainchar.state==0) {
			if(KeyDown(KEY_RIGHT)) {
				mainchar.direction=2;
				mainchar.needupdate=1;
				// check dest tile
				if(mainchar.loc.c<curmap->width) {
					if(curmap->rpg[(mainchar.loc.r)*curmap->width+mainchar.loc.c+1].blocked==0) {
						mainchar.animframe=0;
						mainchar.state=1;
						if(curmap->rpg[(mainchar.loc.r)*curmap->width+mainchar.loc.c+1].event==rpgevent_mapchg) {
							mainchar.mystate=1; // changing map
							tempperson=curmap->rpg[(mainchar.loc.r)*curmap->width+mainchar.loc.c+1].person;
							tempparam=curmap->rpg[(mainchar.loc.r)*curmap->width+mainchar.loc.c+1].param;
						}
					}
				}
			}
		}
		if(mainchar.state==0) {
			if(KeyPressed(KEY_A)) {
				i=mainchar.loc.r*curmap->width+mainchar.loc.c;
				a=0;
				if(mainchar.direction==0 && mainchar.loc.r>0) { i-=curmap->width; a++; }
				if(mainchar.direction==4 && mainchar.loc.r<curmap->height) { i+=curmap->width; a++; }
				if(mainchar.direction==6 && mainchar.loc.c>0) { i--; a++; }
				if(mainchar.direction==2 && mainchar.loc.c<curmap->width) { i++; a++; }
				if(a==1) {
					switch(curmap->rpg[i].event) {
					case rpgevent_gentext: // Ahh, text
						TextStart();
						TextDisp((char *)rpg_genstrings[curmap->rpg[i].param],curmap->rpg[i].person);
						TextEnd();
						break;
					case rpgevent_mapchga:


						break;
					case rpgevent_multitext:
						TextStart();
						for(a=0;a<curmap->rpg[i].person+1;a++) {
							TextDisp((char *)rpg_genstrings[curmap->rpg[i].param+a],2);
							if(a!=curmap->rpg[i].person) TextClear();
						}
						TextEnd();
						break;
					case rpgevent_minigame:
						TextStart();
						a=TextAskYN("Want to play pong??",1);
						if(a==1) {
							TextClear();
							TextDisp("A to start game, B to exit.\nReady?",2);
						}
						TextEnd();
						if(a==1) {
							FadeOutBlack(16);

							
							Pongish();

							// restore current game
							rpg_changemap(curmap,-1);
							FadeInBlack(16);
						}
						break;
					}
				}
			}
		}


		// etc updating
		rpg_mapfollowchar=1; // let interrupt center map for us


		if(mainchar.state>0) mainchar.animframe++;

		if(mainchar.mystate==1) {
			BLDMOD=0x00FF;
			if(mainchar.animframe<8) {
				COLY=mainchar.animframe*2;
			} else if(mainchar.animframe==8) {
				COLY=16;
				rpg_changemap((mapdata *)maps[tempperson],tempparam);
			} else if(mainchar.animframe<16) {
				COLY=(16-mainchar.animframe)*2;
			} else {
				mainchar.mystate=0;
				BLDMOD=0;
			}
		}

	} while(1);
}

// old game loop:
/*	SetupMode();
	LoadMap((mapdata *)&outdoormap,0,0);
	LoadTileSet(outdoormap.ts);
	mainchar.currentmap=(mapdata *)&outdoormap;
	ClearOam();
	for(i=0;i<255;i++) PAL[256+i]=charsprites0_pal[i];
	AddSprite(1,1,30,1,(u32 *)&mainchar);

	s32 myx,myy;
	myx=120;
	myy=80;
	do {
		WaitVblank();
		CenterMap(myx,myy);
		if(KeyDown(KEY_UP)) myy-=2;
		if(KeyDown(KEY_DN)) myy+=2;
		if(KeyDown(KEY_LEFT)) myx-=2;
		if(KeyDown(KEY_RIGHT)) myx+=2;
		if(KeyPressed(KEY_A)) {
			TextStart();
			TextDisp("This is a test of the automated text support routines! Let's see how well they work :)",1);
			TextClear();
			TextDisp("Yet more text - we're just testing it you know....",2);
			TextEnd();
		}


	} while(1);

*/


