#include "gba.h"
#include "krawall.h"

#define t_basetileram 0x0c400


typedef struct tileset_ {
	u16 numtiles, numpals;
	u16 * tiledata;
	u16 * paldata;
} tileset;

typedef struct rpgdata_ {
	u8 blocked;
	u8 event;
	u16 person;
	u32 param;
} rpgdata;

typedef struct mappos_ {
	u16 r, c;
} mappos;

typedef struct entpts_ {
	u32 numpoints;
	mappos pt[];
} entpts;

typedef struct mapdata_ {
	u16 width,height;
	tileset * ts;
	u16 * map;
	rpgdata * rpg;
	entpts * eps;
} mapdata;


// structs for animation data representation
typedef struct chaniminfo_ {
	u16 length;
	u16 starttile;
} chaniminfo;

typedef struct chanimi_ {
	u16 * sprdata;	// sprdata: starts with each direction (8 dirs) then has other stuff...
					// directions start at top and go clockwise
	chaniminfo walk[4]; // up, right, down, left
	chaniminfo other[]; // additional optional animations :)
} chanim; // character animation struct

typedef struct chardata_ { // constant character data
	chanim * anim;
	char * name;
	// put other important data here!
} chardata;

typedef struct charinfo_ { // variable character info
	chardata * chdata; // info about character
	mapdata * currentmap; // where he's at
	mappos loc;	// where in the map
	u16 state;	// character state, what he's doing
	u8 direction; // facing/moving direction
	u8 needupdate; // tells animation system if something has changed since last update
	u16 animframe; // trame in animation that is currently running...
	u16 mystate;	// for local game stuff...
	// add more stuff here
} charinfo;

typedef struct staticspr_ {
	u16 x;
	u16 y;
	u16 size;
	u16 spritenum;
	u16 * tiledata;
} staticspr;

typedef struct sprinfo_ {
	u8 type;
	u8 visible;
	u8 priority;
	u8 id;
	u32 * spr; // link to actual data
	u32 starttilenum;
} sprinfo;



// rpg events defined
// walkover type events
#define rpgevent_none 0
#define rpgevent_mapchg 1
#define rpgevent_custom 2

// press-a at a square events
#define rpgevent_gentext 32
#define rpgevent_mapchga 33
#define rpgevent_multitext 34
#define rpgevent_minigame 35

// car/pedestrian events
#define rpgevent_car 128	// person=car directions it applies to (16 for trucks)
	// param = directions available to go


// rpg data
extern const char * rpg_genstrings[];
extern const char * rpg_gameaskstrings[];

extern const mapdata * maps[];


void rpg_gameloop();
void TextStart();
void TextDisp(char * txt,u16 snd);
u8 TextAskYN(char * txt, u16 snd);
void TextClear();
void TextEnd();



// mapscroll declares

void SetupMode();
void ResetTileSet();
void LoadTileSet(tileset * tset);
void LoadMap(mapdata * map,s32 x,s32 y);
void CenterMap(s32 x, s32 y);

// anim defines

void ClearOam();
void AddSprite(u8 type, u8 id, u8 priority,u8 visible, u32 * dat);
void DeleteSprite(u8 id);
void SetVisible(u8 id, u8 visible);
void SetPriority( u8 id, u8 priority);
// call once a vblank
void UpdateOam();


// kracache declares
void KraPlay(Module * song);
void KraPlaySave(Module * song);
void KraPlaySaved();


// externs
extern u32 mpx,mpy; // mpx,mpy coordinates of top left screen in map

extern const Module * music[];

extern const u16 framestuff_pal[];
extern const u32 framestuff_numtiles;
extern const u8 framestuff_tile[];

extern const u16 charsprites0_pal[];
extern const u32 charsprites0_numtiles;
extern const u8 charsprites0_tile[];

extern const u16 spritedata_pal[];
extern const u32 spritedata_numtiles;
extern const u8 spritedata_tile[];




// maps and tilesets:


extern const mapdata outdoormap;
extern const mapdata aptA1;
extern const mapdata aptA2;
extern const mapdata aptB1;
extern const mapdata aptB2;
extern const mapdata aptC1;
extern const mapdata aptC2;
extern const mapdata arcademap;
extern const mapdata storemap;


extern const tileset outdoor_map;
extern const tileset apartment_map;
extern const tileset arcade_map;
extern const tileset store_map;


// characters


extern const chanim maincharanim;
extern chardata mainchardata;
extern charinfo mainchar;










