// Stephen Stair - graphics routines for Card Party GBA
// this is an #included file, so no need for headers.
// v1.0 2-11-03 Began file
// v1.1 2-13-03 Added support for gfxinit, setpixel, getpiexl, and drawtext.  Sorry no wordwrap / paramaters yet!
// v1.2 2-27-03 Added support for bigger fonts, drawline, some small optimizations.
// v1.3 3-04-03 Added image drawing and palette loading support LoadPal and DrawImage!
#include "gameland.h"

extern const u8 fontdata[];


typedef struct fdata_ {
	u8 numfonts;
	u8 unused[3];
	u32 font_offset[];
} fdata;

typedef struct font_ {
	u8 height;
	s8 pitchH,pitchV;
	u8 unused;
	u8 charwid[256];
	u16 charlut[256];
	union {
		u8 chardata8[65536];
		u16 chardata16[32768];
	};
} fontinfo;




// our local global variables
u32 mygfxmode;
u32 mycurrentpal;
u32 mycolor;


// general

void WaitVblank() {
	while(VCOUNT!=159);
	while(VCOUNT==159);
	//UpdateKeys();
}

asm ( ".POOL" );



int GetTextLength(u32 font, char * fs) {
	fdata * fd;
	fontinfo * fo;
	u32 i,cw;
	u8 c;
	
	fd= (fdata *)fontdata;
	if(font>fd->numfonts) return 0;
	fo=(fontinfo *)(fontdata+fd->font_offset[font]);
	cw=0;
	i=0;
	do {
		c=fs[i++];
		cw+=fo->charwid[c];
		cw+=fo->pitchH;
	} while(fs[i]);
	return cw;
}

int GetCharLength(u32 font, char fs) {
	fdata * fd;
	fontinfo * fo;
	
	fd= (fdata *)fontdata;
	if(font>fd->numfonts) return 0;
	fo=(fontinfo *)(fontdata+fd->font_offset[font]);

	return fo->charwid[(u8)fs]+fo->pitchH;

}


int GetWordLength(u32 font, char * fs) {
	fdata * fd;
	fontinfo * fo;
	u32 i,cw;
	u8 c;
	
	fd= (fdata *)fontdata;
	if(font>fd->numfonts) return 0;
	fo=(fontinfo *)(fontdata+fd->font_offset[font]);
	cw=0;
	i=0;
	do {
		c=fs[i++];
		cw+=fo->charwid[c];
		cw+=fo->pitchH;
	} while(fs[i]!=0 && fs[i]!=' ' && fs[i]!='\n');
	return cw;
}


void TsetPixel(s32 x, s32 y, u32 c) {
	u16 tile;
	if(x<0 || x>224 || y<0 || y>32) return;
	tile=y/8*28+x/8;
	if(x%2==0) {
		VRAM[t_basetileram/2+320+tile*32+(y%8)*4+(x%8)/2]=
			(VRAM[t_basetileram/2+320+tile*32+(y%8)*4+(x%8)/2] & 0xff00) | c;
	} else {
		VRAM[t_basetileram/2+320+tile*32+(y%8)*4+(x%8)/2]=
			(VRAM[t_basetileram/2+320+tile*32+(y%8)*4+(x%8)/2] & 0x00ff) | (c<<8);
	}
}


int TextCharOut(s32 x, s32 y,u32 font, char  fs, u32 color) {
	fdata * fd;
	fontinfo * fo;
	u32 j,h,ex,z;
	u8 c;
	
	fd= (fdata *)fontdata;
	if(font>fd->numfonts) return 0;
	fo=(fontinfo *)(fontdata+fd->font_offset[font]);

	//va_list args;
	//va_start(args,fstr);
	if(fo->height>8) {
		// >8 routine

		c=fs;
		// get character data
		z=fo->charlut[c];
		// draw character
		for(j=0;j<fo->charwid[c];j++) {
			// for each column
			ex=1;
			for(h=0;h<fo->height;h++) {
				if(fo->chardata16[z] & ex) TsetPixel(j+x,y+h,color);
				ex=ex*2;
			}
			z++;
		}
	} else {
		// <=8 routine
		c=fs;
		// get character data
		z=fo->charlut[c];
		// draw character
		for(j=0;j<fo->charwid[c];j++) {
			// for each column
			ex=1;
			for(h=0;h<fo->height;h++) {
				if(fo->chardata8[z] & ex) TsetPixel(x+j,y+h,color);
				ex=ex*2;
			}
			z++;
		}
	}
	return fo->charwid[c] + fo->pitchH ;
}


/*

void TextOut(s32 x, s32 y, u32 width, u32 font, char * fs,u32 color) {
	fdata * fd;
	fontinfo * fo;
	u32 curx,cury,i,j,h,ex,z;
	u8 c;
	
	fd= (fdata *)fontdata;
	if(font>fd->numfonts) return;
	fo=(fontinfo *)(fontdata+fd->font_offset[font]);

	//va_list args;
	//va_start(args,fstr);
	curx=cury=i=0;
	
	if(fo->height>8) {
		// >8 routine
		do {
			c=fs[i++];
			if(curx+fo->charwid[c]>(width-1)) { 
				// go to next line
				cury+=fo->pitchV;
				curx=0;
			}
			// get character data
			z=fo->charlut[c];
			// draw character
			for(j=0;j<fo->charwid[c];j++) {
				// for each column
				ex=1;
				for(h=0;h<fo->height;h++) {
					if(fo->chardata16[z] & ex) TsetPixel(curx+x,cury+y+h,color);
					ex=ex*2;
				}
				z++;
				curx++;
			}
			curx+=fo->pitchH;

		} while(fs[i] && cury<height);



	} else {
		// <=8 routine
		do {
			c=fs[i++];
			if(curx+fo->charwid[c]>(width-1)) { 
				// go to next line
				cury+=fo->pitchV;
				curx=0;
			}
			// get character data
			z=fo->charlut[c];
			// draw character
			for(j=0;j<fo->charwid[c];j++) {
				// for each column
				ex=1;
				for(h=0;h<fo->height;h++) {
					if(fo->chardata8[z] & ex) SetPixel(curx+x,cury+y+h,color);
					ex=ex*2;
				}
				z++;
				curx++;
			}
			curx+=fo->pitchH;

		} while(fs[i] && cury<height);
	}
}

*/

/*
SetPixel_func SetPixel;
GetPixel_func GetPixel;

u32 textmode_tile,textmode_numrows,textmode_numcols;

void SetPixel_16bit(s32 x, s32 y, u32 color) {
	if(x<0 || y<0 || x>239 || y>159) return;
	VRAM[y*240+x]=color;
}
void SetPixel_8bit(s32 x, s32 y, u32 color) {
	if(x<0 || y<0 || x>239 || y>159) return;
	if(x%2) {
		VRAM[(y*120)+(x/2)]=(VRAM[(y*120)+(x/2)]& 0x00ff) | (color << 8);
	} else {
		VRAM[(y*120)+(x/2)]=(VRAM[(y*120)+(x/2)]& 0xff00) | color;
	}
}
void SetPixel_text4(s32 x, s32 y, u32 color) {
	if(x<0 || y<0 || x>textmode_numcols*8 || y>textmode_numrows*8) return;
	switch(x%4) {
	case 0:
		break;
	case 1:
		break;
	case 2:
		break;
	case 3:
		break;
	}
}
void SetPixel_text8(s32 x, s32 y, u32 color) {
	if(x<0 || y<0 || x>textmode_numcols*8 || y>textmode_numrows*8) return;
	VRAM[y*240+x]=color;
}




u32 GetPixel_8bit(s32 x, s32 y)
{
	u16 color;
	color=VRAM[(y*120)+(x/2)];
	if(x%2) color=color>>8;
	return color & 255;
}
u32 GetPixel_16bit(s32 x, s32 y)
{
	u16 color;
	color=VRAM[(y*120)+(x/2)];
	if(x%2) color=color>>8;
	return color & 255;
}
u32 GetPixel_text4(s32 x, s32 y)
{
	u16 color;
	color=VRAM[(y*120)+(x/2)];
	if(x%2) color=color>>8;
	return color & 255;
}
u32 GetPixel_text8(s32 x, s32 y)
{
	u16 color;
	color=VRAM[(y*120)+(x/2)];
	if(x%2) color=color>>8;
	return color & 255;
}




void SetDrawMode(u32 mode) {
	// mode 0=tile4bit,1=tile8bit mode2=8bit, mode3=16bit
	switch(mode) {
	case 0:
//		SetPixel=SetPixel_tile4;
//		GetPixel=GetPixel_tile4;
		break;
	case 1:
//		SetPixel=&SetPixel_tile8;
//		GetPixel=&GetPixel_tile8;
		break;
	case 2:
		SetPixel=&SetPixel_8bit;
		GetPixel=&GetPixel_8bit;
		break;
	case 3:
		SetPixel=&SetPixel_16bit;
		GetPixel=&GetPixel_16bit;
		break;
	}
}



// starttile: offset from start of vram.
// numrows, numcols: 1=1 tile row/col
void SetTileClip(u32 starttile,u32 numrows,u32 numcols) {
	textmode_tile=starttile;
	textmode_numrows=numrows;
	textmode_numcols=numcols;

}

*/



/*
#define sign(x) (((x)==0)?0:(((x)<0)?-1:1))
void DrawLine(s32 x1, s32 y1, s32 x2, s32 y2, u32 color) {
	u32 dy,dx;
	u32 x,y;
	dy=y2-y1;
	dx=x2-x1;
	if(abs(dy)>abs(dx)) {
		// step vertically
		dx=(dx*65536)/abs(dy);
		x=x1<<16;
		for(y=y1;y!=y2;y+=sign(dy)) {
			SetPixel(x>>16,y,color);
			x+=dx;
		}
	} else {
		// step horizontally
		dy=(dy*65536)/abs(dx);
		y=y1<<16;
		for(x=x1;x!=x2;x+=sign(dx)) {
			SetPixel(x,y>>16,color);
			y+=dy;
		}
	}
}
*/